/*
 *
 *  ao_usound.c
 *
 *      Based on ao_oss.c
 *      Original Copyright (C) Aaron Holtzman - May 1999
 *      Modifications Copyright (C) Stan Seibert - July 2000, June 2001
 *      Adapted for usound by Matt Campbell - March 2004
 *
 *  This file is part of libao, a cross-platform library.  See
 *  README for a history of this source code.
 *
 *  libao is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *
 *  libao is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with GNU Make; see the file COPYING.  If not, write to
 *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <stdio.h>
#include <errno.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <math.h>
#include <usound.h>

#include "ao/ao.h"
#include "ao/plugin.h"


static char *ao_usound_options[] = {};
static ao_info ao_usound_info =
{
	AO_TYPE_LIVE,
	"USound audio driver output ",
	"usound",
	"Matt Campbell <mattc@freedombox.cc>",
	"Outputs audio to the Useful Sound Daemon.",
	AO_FMT_NATIVE,
	40,
	ao_usound_options,
	0
};


int ao_plugin_test()
{
	return usound_is_available();
}


ao_info *ao_plugin_driver_info(void)
{
	return &ao_usound_info;
}


int ao_plugin_device_init(ao_device *device)
{
	device->internal = NULL;
	return 1;
}

int ao_plugin_set_option(ao_device *device, const char *key, const char *value)
{
	/* No options yet. */
	return 1;
}


/*
 * open the audio device for writing to
 */
int ao_plugin_open(ao_device *device, ao_sample_format *format)
{
	int sample_bytes;
	usound_output_t stream = NULL;

	if (format->channels != 1 && format->channels != 2) {
		fprintf(stderr,"libao - Unsupported number of channels: %d.",
			format->channels);
		return 0;
	}

	switch (format->bits)
	{
	case 8: sample_bytes = 1;
		break;
        case 16: sample_bytes = 2;
	        device->driver_byte_format = AO_FMT_NATIVE;
	        break;
	default:fprintf(stderr,"libao - Unsupported number of bits: %d.",
			format->bits);
		return 0;
	}

	stream = usound_open_output(sample_bytes, format->channels,
				    format->rate);

	if (stream == NULL) {
		return 0;
	} else {
		usound_play (stream);
		device->internal = stream;
		return 1;
	}
}

/*
 * play the sample to the already opened file descriptor
 */
int ao_plugin_play(ao_device *device, const char *output_samples, 
		uint_32 num_bytes)
{
	usound_output_t stream = (usound_output_t) device->internal;

	if (usound_write(stream, output_samples, num_bytes) < 0)
		return 0;
	else
		return 1;
}


int ao_plugin_close(ao_device *device)
{
	usound_output_t stream = (usound_output_t) device->internal;
	usound_close_output(stream);

	return 1;
}


void ao_plugin_device_clear(ao_device *device)
{
}
