/*
 * ao_usound - USound audio output driver for MPlayer
 * Matt Campbell <mattcampbell@pobox.com>
 *
 * Based on ao_arts - aRts audio output driver for MPlayer
 * Michele Balistreri <brain87@gmx.net>
 *
 * This driver is distribuited under terms of GPL
 *
 */

#include <stdio.h>
#include <usound.h>

#include "audio_out.h"
#include "audio_out_internal.h"
#include "afmt.h"
#include "../config.h"
#include "../mp_msg.h"

static usound_output_t stream;

static ao_info_t info =
{
    "USound audio output",
    "usound",
    "Matt Campbell <mattcampbell@pobox.com>",
    ""
};

LIBAO_EXTERN(usound)

static int control(int cmd, void *arg)
{
	return(CONTROL_UNKNOWN);
}

static int init(int rate_hz, int channels, int format, int flags)
{
	int err, sample_bytes;
	/* Let MPlayer resample if necessary. */
	rate_hz = 44100;

	/*
	 * usound supports 8bit unsigned and 16bit signed sample formats
	 *
	 * Unsupported formats are translated to one of these two formats
	 * using mplayer's audio filters.
	 */
	switch (format) {
	case AFMT_U8:
	case AFMT_S8:
	    format = AFMT_U8;
	    sample_bytes = 1;
	    break;
	default:
	    format = AFMT_S16_NE;
	    sample_bytes = 2;
	    break;
	}

	ao_data.format = format;
	ao_data.channels = channels;
	ao_data.samplerate = rate_hz;
	ao_data.bps = (rate_hz*channels);

	if(format != AFMT_U8 && format != AFMT_S8)
		ao_data.bps*=2;

	stream = usound_open_output(sample_bytes, channels, rate_hz);

	if(stream == 0) {
		mp_msg(MSGT_AO, MSGL_ERR, "AO: [usound] Unable to open a stream\n");
		return 0;
	}

	ao_data.buffersize = usound_get_output_buffer_size(stream);
	mp_msg(MSGT_AO, MSGL_INFO, "AO: [usound] Stream opened\n");

	mp_msg(MSGT_AO, MSGL_INFO,"AO: [usound] buffer size: %d\n",
	    ao_data.buffersize);
	usound_play(stream);

	return 1;
}

static void uninit()
{
	usound_reset_output(stream);
	usound_close_output(stream);
}

static int play(void* data,int len,int flags)
{
	return usound_write(stream, data, len);
}

static void audio_pause()
{
	usound_stop(stream);
}

static void audio_resume()
{
	usound_play(stream);
}

static void reset()
{
	usound_reset_output(stream);
}

static int get_space()
{
	return usound_get_output_space(stream);
}

static float get_delay()
{
	return ((float) (ao_data.buffersize - 
		usound_get_output_space(stream))) / ((float) ao_data.bps);
}

