#include <pth.h>
#include <stdlib.h>
#include "control.h"
#include "device.h"
#include "mixer.h"
#include "output_stream.h"
#include "ringbuffer.h"
#include "unix_listen.h"
#include "unix_serve.h"

static void
_handler (int sock, void *data)
{
  struct output_stream *s = NULL;
  int owns_stream = FALSE;
  int index = -1;

  while (1)
    {
      unsigned char cmd = 0;
      int rv = pth_read (sock, &cmd, sizeof (cmd));

      if (rv <= 0)
	break;

      switch (cmd)
	{
	case 0:
	  {
	    int tmp =
	      device_get_sample_bytes () | (device_get_channels () << 16);
	    pth_write (sock, &tmp, sizeof (tmp));
	    tmp = device_get_rate ();
	    pth_write (sock, &tmp, sizeof (tmp));
	    tmp = device_get_fragment_size ();
	    pth_write (sock, &tmp, sizeof (tmp));
	    break;
	  }
	case 1:
	  {
	    int res = -1;

	    if (!owns_stream)
	      {
		if (s != NULL)
		  {
		    output_stream_unref (s);
		    s = NULL;
		  }

		s = output_stream_create ();
		output_stream_ref (s);
		res = index = mixer_add_stream (s);
		owns_stream = TRUE;
	      }

	    pth_write (sock, &res, sizeof (res));
	    break;
	  }
	case 2:
	  {
	    int res = -1;
	    int new_index = -1;
	    struct output_stream *new_s = NULL;
	    pth_read (sock, &new_index, sizeof (new_index));

	    if (!owns_stream && new_index >= 0)
	      {
		new_s = mixer_get_stream (new_index);

		if (new_s != NULL)
		  {
		    if (s != NULL)
		      {
			output_stream_unref (s);
			s = NULL;
		      }

		    s = new_s;
		    output_stream_ref (s);
		    index = new_index;
		    res = 0;
		  }
	      }

	    pth_write (sock, &res, sizeof (res));
	    break;
	  }
	case 3:
	  {
	    int res = -1;

	    if (s != NULL)
	      {
		s->playing = TRUE;
		res = 0;
	      }

	    pth_write (sock, &res, sizeof (res));
	    break;
	  }
	case 4:
	  {
	    int res = -1;

	    if (s != NULL)
	      {
		s->playing = FALSE;
		res = 0;
	      }

	    pth_write (sock, &res, sizeof (res));
	    break;
	  }
	case 5:
	  {
	    int res = -1;

	    if (s != NULL)
	      {
		output_stream_sync (s);
		res = 0;
	      }

	    pth_write (sock, &res, sizeof (res));
	    break;
	  }
	case 6:
	  {
	    int res = -1;

	    if (s != NULL)
	      {
		output_stream_reset (s);
		res = 0;
	      }

	    pth_write (sock, &res, sizeof (res));
	    break;
	  }
	case 7:
	  {
	    int res = -1;

	    if (s != NULL)
	      res = s->volume;

	    pth_write (sock, &res, sizeof (res));
	    break;
	  }
	case 8:
	  {
	    int res = -1;

	    if (s != NULL)
	      {
		pth_read (sock, &s->volume, sizeof (s->volume));
		res = 0;
	      }

	    pth_write (sock, &res, sizeof (res));
	    break;
	  }
	case 9:
	  {
	    int res = -1;

	    if (s != NULL)
	      res = 0;

	    pth_write (sock, &res, sizeof (res));

	    if (s != NULL)
	      {
		long long bytes_played;

		if (s->format_ratio == 1.0)
		  bytes_played = s->bytes_played;
		else
		  {
		    bytes_played =
		      (long long) (s->bytes_played * s->format_ratio + 16);

		    if (bytes_played > s->bytes_received)
		      bytes_played = s->bytes_received;
		  }

		pth_write (sock, &bytes_played, sizeof (bytes_played));
	      }

	    break;
	  }
	case 10:
	  {
	    int res = -1;

	    if (s != NULL)
	      res = s->playing ? 1 : 0;

	    pth_write (sock, &res, sizeof (res));
	    break;
	  }
	case 11:
	  {
	    int res = -1;

	    if (s != NULL)
	      {
		res = s->rbuf.bufferSize;

		if (s->format_ratio != 1.0)
		  res = (int) (res * s->format_ratio);
	      }

	    pth_write (sock, &res, sizeof (res));
	    break;
	  }
	case 12:
	  {
	    int res = -1;

	    if (s != NULL)
	      {
		res = RingBuffer_GetWriteAvailable (&s->rbuf);

		if (s->format_ratio != 1.0)
		  res = (int) (res * s->format_ratio);
	      }

	    pth_write (sock, &res, sizeof (res));
	    break;
	  }
	case 13:
	  {
	    int res = -1;

	    if (s != NULL)
	      {
		int buffer_size;
		pth_read (sock, &buffer_size, sizeof (buffer_size));
		output_stream_set_buffer_size (s, buffer_size);
		res = 0;
	      }

	    pth_write (sock, &res, sizeof (res));
	    break;
	  }
	default:
	  {
	    int tmp = -1;
	    pth_write (sock, &tmp, sizeof (tmp));
	    break;
	  }
	}
    }

  if (owns_stream)
    {
      mixer_remove_stream (index);
      output_stream_close (s);
    }

  if (s != NULL)
    {
      output_stream_unref (s);
      s = NULL;
    }
}

static void *
_server_thread (void *_arg)
{
  int sock = unix_listen ("/dev/usound/ctl", 3);
  unix_serve (sock, _handler, NULL);
  return NULL;
}

void
control_start (void)
{
  pth_spawn (PTH_ATTR_DEFAULT, _server_thread, NULL);
}
