#include <fcntl.h>
#include <sys/ioctl.h>
#include <sys/soundcard.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <unistd.h>
#include "device.h"

static int _fd = -1;
static int _sample_bytes = -1;
static int _channels = -1;
static int _rate = -1;
static int _fragment_size = -1;

int
device_open (int sample_bytes, int channels, int rate, int fragment_size_exp)
{
  int format = (sample_bytes == 8) ? AFMT_U8 : AFMT_S16_NE;
  int stereo = (channels == 2) ? 1 : 0;
  int fragment = 0x00020000 | fragment_size_exp;
  _sample_bytes = sample_bytes;
  _channels = channels;
  _rate = rate;

  if (_fd != -1)
    close (_fd);

  _fd = open ("/dev/dsp", O_RDWR);

  if (_fd != -1)
    {
      if (ioctl (_fd, SNDCTL_DSP_SETFRAGMENT, &fragment) == -1 ||
	  ioctl (_fd, SNDCTL_DSP_SAMPLESIZE, &format) == -1 ||
	  ioctl (_fd, SNDCTL_DSP_STEREO, &stereo) == -1 ||
	  ioctl (_fd, SNDCTL_DSP_SPEED, &rate) == -1 ||
	  ioctl (_fd, SNDCTL_DSP_GETBLKSIZE, &_fragment_size) == -1)
	{
	  close (_fd);
	  _fd = -1;
	}
    }

  return _fd;
}

void
device_close (void)
{
  if (_fd != -1)
    {
      close (_fd);
      _fd = -1;
    }
}

int
device_get_fd (void)
{
  return _fd;
}

int
device_get_sample_bytes (void)
{
  return _sample_bytes;
}

int
device_get_channels (void)
{
  return _channels;
}

int
device_get_rate (void)
{
  return _rate;
}

int
device_get_fragment_size (void)
{
  return _fragment_size;
}

int
device_get_writable_fragments (void)
{
  audio_buf_info info;

  if (ioctl (_fd, SNDCTL_DSP_GETOSPACE, &info) == -1)
    return -1;
  else
    return info.fragments;
}

int
device_get_output_delay (void)
{
  int delay;

  if (ioctl (_fd, SNDCTL_DSP_GETODELAY, &delay) == -1)
    return -1;
  else
    return delay;
}
