#include <signal.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include "device.h"
#include "dumb_resample.h"
#include "input.h"
#include "unix_listen.h"
#include "unix_serve_simple.h"

#define BUFFER_SIZE 1024

static int
_read_samples (int fd, sample_t *buf32)
{
  short buf16[BUFFER_SIZE];
  int rv;
  fd_set rfds;
  FD_ZERO (&rfds);
  FD_SET (fd, &rfds);

  rv = read (fd, buf16, sizeof (buf16));

  if (rv > 0)
    {
      int i;
      int channels = device_get_channels ();
      rv /= (sizeof (short) * channels);

      for (i = 0; i < rv; i++)
	buf32[i] = buf16[i * channels];
    }

  return rv;
}

static void
_pickup (DUMB_RESAMPLER *resampler, void *data)
{
  int fd = (int) data;
  int rv = _read_samples (fd, resampler->src);

  if (rv <= 0)
    resampler->dir = 0;
  else
    {
      resampler->pos = resampler->subpos = resampler->start = 0;
      resampler->end = rv;
      resampler->dir = 1;
    }
}

static inline int
_clamp (int min, int val, int max)
{
  if (val < min)
    return min;
  else if (val > max)
    return max;
  else
    return val;
}

static void
_handler (int sock, void *data)
{
  int fd = device_get_fd ();
  int rate, rv, i;
  DUMB_RESAMPLER resampler;
  sample_t inbuf[BUFFER_SIZE];
  sample_t outbuf32[BUFFER_SIZE];
  short outbuf16[BUFFER_SIZE];
  float delta;
  read (sock, &rate, sizeof (rate));
  rv = _read_samples (fd, inbuf);
  dumb_reset_resampler (&resampler, inbuf, 0, 0, rv);
  resampler.pickup = _pickup;
  resampler.pickup_data = (void *) fd;
  delta = (float) device_get_rate () / rate;

  while (1)
    {
      memset (outbuf32, 0, sizeof (sample_t) * BUFFER_SIZE);
      rv = dumb_resample (&resampler, outbuf32, BUFFER_SIZE, 1.0, delta);

      for (i = 0; i < rv; i++)
	outbuf16[i] = _clamp (-32768, outbuf32[i], 32767);

      if (rv > 0)
	{
	  rv = write (sock, outbuf16, rv * sizeof (short));

	  if (rv <= 0)
	    break;
	}
    }
}

void
input_start (void)
{
  int sock;
  signal (SIGCHLD, SIG_IGN);

  if (fork () != 0)
    return;

  sock = unix_listen ("/dev/usound/in", 3);
  signal (SIGPIPE, SIG_IGN);
  unix_serve_simple (sock, _handler, NULL);
}
