#include <alloca.h>
#include <pth.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/select.h>
#include <unistd.h>
#include "device.h"
#include "mixer.h"
#include "output_stream.h"

static struct output_stream **_streams = NULL;
static int _max_streams = 0;
static int _active_streams = 0;

/* XXX: Assumes 16-bit mixing for now. */
void
mixer_main (void)
{
  int fd = device_get_fd ();
  int bufsize = device_get_fragment_size ();
  short *buffer = (short *) alloca (bufsize);
  short *tmpbuf = (short *) alloca (bufsize);

  while (1)
    {
      int i;
      fd_set wfds;
      FD_ZERO (&wfds);
      FD_SET (fd, &wfds);
      pth_select (fd + 1, NULL, &wfds, NULL, NULL);
      memset (buffer, 0, bufsize);

      for (i = 0; i < _max_streams; i++)
	{
	  struct output_stream *s = _streams[i];
	  int bytes;
	  int samples;
	  int volume;
	  int j;

	  if (s == NULL || !s->playing)
	    continue;

	  bytes = output_stream_read (s, tmpbuf, bufsize);
	  s->bytes_played += bytes;
	  samples = bytes / 2;
	  volume = s->volume;

	  for (j = 0; j < samples; j++)
	    {
	      int sample = buffer[j] + (tmpbuf[j] * volume >> 16);

	      if (sample < -32768)
		buffer[j] = -32768;
	      else if (sample > 32767)
		buffer[j] = 32767;
	      else
		buffer[j] = sample;
	    }

	  if (bytes < bufsize)
	    output_stream_sync_notify (s);
	}

      write (fd, buffer, bufsize);
    }
}

void
mixer_init (void)
{
  _streams =
    (struct output_stream **) malloc (sizeof (struct output_stream *) * 16);
  _max_streams = 16;
  memset (_streams, 0, sizeof (struct output_stream *) * 16);
}

int
mixer_add_stream (struct output_stream *s)
{
  int i;

  if (_active_streams == _max_streams)
    {
      _streams =
	(struct output_stream **) realloc (_streams,
					   sizeof (struct output_stream *) *
					   _max_streams + 1);
      _streams[_max_streams++] = NULL;
    }

  for (i = 0; i < _max_streams; i++)
    {
      if (_streams[i] == NULL)
	{
	  _streams[i] = s;
	  output_stream_ref (s);
	  _active_streams++;
	  return i;
	}
    }

  return -1;
}

struct output_stream *
mixer_get_stream (int i)
{
  if (i < _max_streams)
    return _streams[i];
  else
    return NULL;
}

void
mixer_remove_stream (int i)
{
  if (i < _max_streams && _streams[i] != NULL)
    {
      output_stream_unref (_streams[i]);
      _streams[i] = NULL;
      _active_streams--;
    }
}
