#ifndef OUTPUT_STREAM_H
#define OUTPUT_STREAM_H

#include <pth.h>
#include "ringbuffer.h"

struct output_stream
{
  RingBuffer rbuf;
  pth_mutex_t mutex;
  pth_cond_t cond;
  pth_cond_t sync_cond;
  int volume;
  int playing;
  int ref_count;
  int resetting;
  int closing;
  long long bytes_received;
  long long bytes_played;
  int done_writing;
  double format_ratio;
};

struct output_stream *output_stream_create (void);
void output_stream_destroy (struct output_stream *s);
int output_stream_read (struct output_stream *s, void *buf, int bytes);
void output_stream_write (struct output_stream *s, void *buf, int bytes);
void output_stream_sync (struct output_stream *s);
void output_stream_sync_notify (struct output_stream *s);
void output_stream_ref (struct output_stream *s);
void output_stream_unref (struct output_stream *s);
void output_stream_reset (struct output_stream *s);
void output_stream_close (struct output_stream *s);
void output_stream_set_buffer_size (struct output_stream *s, int buffer_size);

#endif /* not OUTPUT_STREAM_H */
