#include <sys/socket.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/un.h>
#include <unistd.h>
#include "unix_listen.h"

int
unix_listen (const char *path, int backlog)
{
  struct sockaddr_un sun;
  int sock = -1;

  /* Remove the file if it already exists. */
  unlink (path);

  /* Fill the address structure. */
  memset (&sun, 0, sizeof (sun));
  sun.sun_family = AF_UNIX;
  strcpy (sun.sun_path, path);

  /* Create the socket. */
  sock = socket (AF_UNIX, SOCK_STREAM, 0);

  if (sock == -1)
    return -1;

  /* Bind the address to the socket. */
  if (bind (sock, (struct sockaddr *) &sun, sizeof (sun)) == -1)
    {
      close (sock);
      return -1;
    }

  /* Set the backlog and start listening. */
  if (listen (sock, backlog) == -1)
    {
      close (sock);
      return -1;
    }

  chmod (path, 0666);
  return sock;
}
