#include <pth.h>
#include <sched.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <unistd.h>
#include "constants.h"
#include "control.h"
#include "device.h"
#include "input.h"
#include "mixer.h"
#include "output.h"

int
main (void)
{
  pth_attr_t attr;
  struct sched_param schp = {0};

  if (getuid () != 0)
    {
      fputs ("usound: must be run as root\n", stderr);
      return 1;
    }

  mkdir ("/dev/usound", 0755);

  if (device_open (DEFAULT_SAMPLE_BYTES, DEFAULT_CHANNELS, DEFAULT_RATE,
		   DEFAULT_FRAGMENT_SIZE_EXP) < 0)
    {
      perror ("usound: unable to open device");
      exit (1);
    }

  input_start ();
  schp.sched_priority = sched_get_priority_max (SCHED_RR);
  sched_setscheduler (0, SCHED_RR, &schp);
  signal (SIGPIPE, SIG_IGN);
  pth_init ();
  attr = pth_attr_of (pth_self ());
  pth_attr_set (attr, PTH_ATTR_JOINABLE, FALSE);
  mixer_init ();
  control_start ();
  output_start ();
  mixer_main ();
  return 0;
}
